@@ -1,19 +1,3 @@
-# Downward Lab uses the Lab package to conduct experiments with the
-# Fast Downward planning system.
-#
-# This program is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
-
 """
 A module for running Fast Downward experiments.
 """
@@ -42,32 +26,40 @@ class FastDownwardRun(Run):
         self.algo = algo
         self.task = task
 
-        self._set_properties()
+        self.driver_options = algo.driver_options[:]
 
-        # Linking to instead of copying the PDDL files makes building
-        # the experiment twice as fast.
-        self.add_resource("domain", self.task.domain_file, "domain.pddl", symlink=True)
-        self.add_resource(
-            "problem", self.task.problem_file, "problem.pddl", symlink=True
-        )
+        if self.task.domain_file is None:
+            self.add_resource("task", self.task.problem_file, "task.sas", symlink=True)
+            input_files = ["{task}"]
+            # Without PDDL input files, we can't validate the solution.
+            self.driver_options.remove("--validate")
+        else:
+            self.add_resource(
+                "domain", self.task.domain_file, "domain.pddl", symlink=True
+            )
+            self.add_resource(
+                "problem", self.task.problem_file, "problem.pddl", symlink=True
+            )
+            input_files = ["{domain}", "{problem}"]
 
         self.add_command(
             "planner",
             [tools.get_python_executable()]
             + ["{" + _get_solver_resource_name(algo.cached_revision) + "}"]
-            + algo.driver_options
-            + ["{domain}", "{problem}"]
+            + self.driver_options
+            + input_files
             + algo.component_options,
         )
 
+        self._set_properties()
+
     def _set_properties(self):
         self.set_property("algorithm", self.algo.name)
         self.set_property("repo", self.algo.cached_revision.repo)
         self.set_property("local_revision", self.algo.cached_revision.local_rev)
         self.set_property("global_revision", self.algo.cached_revision.global_rev)
-        self.set_property("revision_summary", self.algo.cached_revision.summary)
         self.set_property("build_options", self.algo.cached_revision.build_options)
-        self.set_property("driver_options", self.algo.driver_options)
+        self.set_property("driver_options", self.driver_options)
         self.set_property("component_options", self.algo.component_options)
 
         for key, value in self.task.properties.items():
@@ -104,13 +96,12 @@ class FastDownwardExperiment(Experiment):
     .. note::
 
         To build the experiment, execute its runs and fetch the results,
-        add the following steps (previous Lab versions added these steps
-        automatically):
+        add the following steps:
 
         >>> exp = FastDownwardExperiment()
-        >>> exp.add_step('build', exp.build)
-        >>> exp.add_step('start', exp.start_runs)
-        >>> exp.add_fetcher(name='fetch')
+        >>> exp.add_step("build", exp.build)
+        >>> exp.add_step("start", exp.start_runs)
+        >>> exp.add_fetcher(name="fetch")
 
     """
 
@@ -182,31 +173,32 @@ class FastDownwardExperiment(Experiment):
         return tasks
 
     def add_suite(self, benchmarks_dir, suite):
-        """Add benchmarks to the experiment.
+        """Add PDDL or SAS+ benchmarks to the experiment.
 
-        *benchmarks_dir* must be a path to a benchmark directory. It
-        must contain domain directories, which in turn hold PDDL files.
+        *benchmarks_dir* must be a path to a benchmark directory. It must
+        contain domain directories, which in turn hold PDDL or SAS+ files.
 
         *suite* must be a list of domain or domain:task names. ::
 
-            exp.add_suite(benchmarks_dir, ["depot", "gripper"])
-            exp.add_suite(benchmarks_dir, ["gripper:prob01.pddl"])
+            >>> benchmarks_dir = os.environ["DOWNWARD_BENCHMARKS"]
+            >>> exp = FastDownwardExperiment()
+            >>> exp.add_suite(benchmarks_dir, ["depot", "gripper"])
+            >>> exp.add_suite(benchmarks_dir, ["gripper:prob01.pddl"])
+            >>> exp.add_suite(benchmarks_dir, ["rubiks-cube:p01.sas"])
 
         One source for benchmarks is
-        https://github.com/aibasel/downward-benchmarks. After cloning
-        the repo, you can generate suites with the ``suites.py``
-        script. We recommend using the suite ``optimal_strips`` for
-        optimal planning and ``satisficing`` for satisficing planning::
+        https://github.com/aibasel/downward-benchmarks. After cloning the
+        repo, you can generate suites with the ``suites.py`` script. We
+        recommend using the suite ``optimal_strips`` for optimal STRIPS planners
+        and ``satisficing`` for satisficing planners::
 
             # Create standard optimal planning suite.
             $ path/to/downward-benchmarks/suites.py optimal_strips
             ['airport', ..., 'zenotravel']
 
-        You can copy the generated list into your experiment script::
+        Then you can copy the generated list into your experiment script::
 
-            >>> benchmarks_dir = REPO = os.environ["DOWNWARD_BENCHMARKS"]
-            >>> exp = FastDownwardExperiment()
-            >>> exp.add_suite(benchmarks_dir, ['airport', 'zenotravel'])
+            >>> exp.add_suite(benchmarks_dir, ["airport", "zenotravel"])
 
         """
         if isinstance(suite, str):
@@ -236,7 +228,7 @@ class FastDownwardExperiment(Experiment):
         *repo* must be a path to a Fast Downward repository.
 
         *rev* must be a valid revision in the given repository (e.g.,
-        ``"default"``, ``"tip"``, ``"issue123"``).
+        ``"e9c2370e6"``, ``"my-branch"``, ``"issue123"``).
 
         *component_options* must be a list of strings. By default these
         options are passed to the search component. Use
@@ -244,18 +236,18 @@ class FastDownwardExperiment(Experiment):
         ``"--search-options"`` within the component options to override
         the default for the following options, until overridden again.
 
-        If given, *build_options* must be a list of strings. They will
-        be passed to the ``build.py`` script. Options can be build names
-        (e.g., ``"release32"``, ``"debug64"``), ``build.py`` options
-        (e.g., ``"--debug"``) or options for Make. If *build_options* is
-        omitted, the ``"release32"`` version is built.
+        If given, *build_options* must be a list of strings. They will be
+        passed to the ``build.py`` script. Options can be build names
+        (e.g., ``"releasenolp"``), ``build.py`` options (e.g.,
+        ``"--debug"``) or options for Make. If *build_options* is omitted,
+        the ``"release"`` version is built.
 
-        If given, *driver_options* must be a list of strings. They will
-        be passed to the ``fast-downward.py`` script. See
-        ``fast-downward.py --help`` for available options. The list is
-        always prepended with ``["--validate", "--overall-time-limit",
-        "30m", "--overall-memory-limit', "3584M"]``. Specifying custom
-        limits overrides the default limits.
+        If given, *driver_options* must be a list of strings. They will be
+        passed to the ``fast-downward.py`` script. See ``fast-downward.py
+        --help`` for available options. The list is always prepended with
+        ``["--validate", "--overall-time-limit", "30m",
+        "--overall-memory-limit', "3584M"]``. Specifying custom limits
+        overrides the default limits.
 
         Example experiment setup:
 
@@ -264,38 +256,37 @@ class FastDownwardExperiment(Experiment):
         >>> exp = FastDownwardExperiment()
         >>> repo = os.environ["DOWNWARD_REPO"]
         >>> vcs = get_version_control_system(repo)
-        >>> rev = "default" if vcs == MERCURIAL else "master"
+        >>> rev = "default" if vcs == MERCURIAL else "main"
 
-        Test iPDB in the latest revision on the default branch:
+        Run iPDB using the latest revision on the main branch:
 
-        >>> exp.add_algorithm(
-        ...     "ipdb", repo, rev,
-        ...     ["--search", "astar(ipdb())"])
+        >>> exp.add_algorithm("ipdb", repo, rev, ["--search", "astar(ipdb())"])
 
         Run blind search in debug mode:
 
         >>> exp.add_algorithm(
-        ...     "blind", repo, rev,
+        ...     "blind",
+        ...     repo,
+        ...     rev,
         ...     ["--search", "astar(blind())"],
         ...     build_options=["--debug"],
-        ...     driver_options=["--debug"])
-
-        Run FF in 64-bit mode:
-
-        >>> exp.add_algorithm(
-        ...     "ff", repo, rev,
-        ...     ["--search", "lazy_greedy([ff()])"],
-        ...     build_options=["release64"],
-        ...     driver_options=["--build", "release64"])
+        ...     driver_options=["--debug"],
+        ... )
 
         Run LAMA-2011 with custom planner time limit:
 
         >>> exp.add_algorithm(
-        ...     "lama", repo, rev,
+        ...     "lama",
+        ...     repo,
+        ...     rev,
         ...     [],
         ...     driver_options=[
-        ...         "--alias", "seq-saq-lama-2011",
-        ...         "--overall-time-limit", "5m"])
+        ...         "--alias",
+        ...         "seq-saq-lama-2011",
+        ...         "--overall-time-limit",
+        ...         "5m",
+        ...     ],
+        ... )
 
         """
         if not isinstance(name, str):
@@ -319,8 +310,7 @@ class FastDownwardExperiment(Experiment):
         for algo in self._algorithms.values():
             if algorithm == algo:
                 logging.critical(
-                    "Algorithms {algo.name} and {algorithm.name} are "
-                    "identical.".format(**locals())
+                    f"Algorithms {algo.name} and {algorithm.name} are identical."
                 )
         self._algorithms[name] = algorithm
 
