@@ -1,19 +1,3 @@
-# Downward Lab uses the Lab package to conduct experiments with the
-# Fast Downward planning system.
-#
-# This program is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
-
 """
 Fast Downward exit codes and their meaning. They are documented at
 http://www.fast-downward.org/ExitCodes.
@@ -84,5 +68,5 @@ def get_outcome(exitcode, use_legacy_exit_codes):
     elif not use_legacy_exit_codes and exitcode in EXITCODE_TO_OUTCOME:
         return EXITCODE_TO_OUTCOME[exitcode]
     else:
-        msg = "exitcode-{exitcode}".format(**locals())
+        msg = f"exitcode-{exitcode}"
         return Outcome(exitcode, msg, explained=False, min_wins=True)
