@@ -1,19 +1,3 @@
-# Downward Lab uses the Lab package to conduct experiments with the
-# Fast Downward planning system.
-#
-# This program is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
-
 import logging
 import os.path
 
@@ -29,16 +13,16 @@ class ScatterPgfplots:
             options["xmax"] = report.x_upper
         if report.y_upper is not None:
             options["ymax"] = report.y_upper
-        lines.append("\\begin{axis}[%s]" % cls._format_options(options))
+        lines.append(f"\\begin{{axis}}[{cls._format_options(options)}]")
         for category, coords in sorted(report.categories.items()):
-            plot = {"only marks": True}
             lines.append(
                 "\\addplot+[{}] coordinates {{\n{}\n}};".format(
-                    cls._format_options(plot), " ".join(str(c) for c in coords)
+                    cls._format_options({"only marks": True}),
+                    " ".join(str(c) for c in coords),
                 )
             )
             if category:
-                lines.append("\\addlegendentry{%s}" % category)
+                lines.append(f"\\addlegendentry{{{category}}}")
             elif report.has_multiple_categories():
                 # None is treated as the default category if using multiple
                 # categories. Add a corresponding entry to the legend.
@@ -48,16 +32,16 @@ class ScatterPgfplots:
             # Add black line at y=1.
             line_min, line_max = cls._get_supported_range(options["xmode"])
             lines.append(
-                "\\draw[color=black] (axis cs:{line_min},1) -- "
-                "(axis cs:{line_max},1);".format(**locals())
+                f"\\draw[color=black] (axis cs:{line_min},1) -- "
+                f"(axis cs:{line_max},1);"
             )
         if report.plot_diagonal_line:
             # Add black diagonal line.
             assert options["xmode"] == options["ymode"]
             line_min, line_max = cls._get_supported_range(options["xmode"])
             lines.append(
-                "\\draw[color=black] (axis cs:{line_min},{line_min}) -- "
-                "(axis cs:{line_max},{line_max});".format(**locals())
+                f"\\draw[color=black] (axis cs:{line_min},{line_min}) -- "
+                f"(axis cs:{line_max},{line_max});"
             )
 
         lines.append("\\end{axis}")
@@ -86,7 +70,7 @@ class ScatterPgfplots:
         )
         tools.makedirs(os.path.dirname(filename))
         tools.write_file(filename, "\n".join(lines))
-        logging.info("Wrote file://%s" % filename)
+        logging.info(f"Wrote file://{filename}")
 
     @classmethod
     def _get_axis_options(cls, report):
@@ -104,8 +88,8 @@ class ScatterPgfplots:
         figsize = report.matplotlib_options.get("figure.figsize")
         if figsize:
             width, height = figsize
-            axis["width"] = "%.2fin" % width
-            axis["height"] = "%.2fin" % height
+            axis["width"] = f"{width:.2f}in"
+            axis["height"] = f"{height:.2f}in"
 
         if report.has_multiple_categories():
             axis["legend style"] = cls._format_options(
@@ -124,8 +108,8 @@ class ScatterPgfplots:
                 opts.append(key)
             elif isinstance(value, str):
                 if " " in value or "=" in value:
-                    value = "{%s}" % value
-                opts.append("{}={}".format(key, value.replace("_", "-")))
+                    value = f"{{{value}}}"
+                opts.append(f"{key}={value.replace('_', '-')}")
             else:
                 opts.append(f"{key}={value}")
         return ", ".join(opts)
