@@ -1,20 +1,4 @@
 #! /usr/bin/env python
-#
-# Downward Lab uses the Lab package to conduct experiments with the
-# Fast Downward planning system.
-#
-# This program is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 """
 Regular expressions and functions for parsing single-search runs of Fast Downward.
@@ -28,7 +12,7 @@ from lab.parser import Parser
 
 
 def _get_states_pattern(attribute, name):
-    return (attribute, r"{name} (\d+) state\(s\)\.".format(**locals()), int)
+    return (attribute, rf"{name} (\d+) state\(s\)\.", int)
 
 
 PATTERNS = [
@@ -55,13 +39,14 @@ PATTERNS = [
 def check_single_search(content, props):
     if "Cumulative statistics:" in content:
         props.add_unexplained_error(
-            "single-search parser can't be used for iterated search"
+            "Single-search parser can't be used for iterated search."
         )
     for _, pattern, _ in PATTERNS:
         results = re.findall(pattern, content)
         if len(results) > 1:
             props.add_unexplained_error(
-                "single-search parser can't be used for anytime planner"
+                f"Found multiple occurences of {pattern} in logfile. "
+                f"Single-search parser can't be used for anytime planner."
             )
 
 
